import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;
import javax.swing.Timer;
import javax.swing.filechooser.FileFilter;

/**
 * FittsTiltTrace - utility software to plot the trace data in the sd3 files created by FittsTilt.
 * <p>
 * 
 * @author Scott MacKenzie, 2011
 */
public class FittsTiltTrace
{
	public static void main(String[] args)
	{
		if (args.length > 0)
		{
			System.out.println("Usage: FittsTiltTrace (see API for details)");
			System.exit(0);
		}

		// use look and feel for my system (Win32)
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e)
		{
		}

		String tmp = "FittsTiltTrace.sd3"; // default filename
		FittsTiltTraceFrame frame = new FittsTiltTraceFrame(tmp);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setTitle("FittsTiltTrace");
		frame.pack();
		frame.setVisible(true);
	}
}

class FittsTiltTraceFrame extends JFrame implements ActionListener, KeyListener, ItemListener
{
	// the following avoids a "warning" with Java 1.5.0 complier (?)
	static final long serialVersionUID = 42L;
	
	final String[] SCALE_OPTIONS = { "0.4", "0.6", "0.8", "1.0", "1.2", "1.4", "1.6" };
	final int DEFAULT_INDEX = 3; // "1.0" (adjust as necessary)

	static int sequenceCounter = 1;
	static int trialCounter = 1;

	private TracePanel tp;

	private JTextField file;
	private JButton browse;
	private JButton next;
	private JButton previous;
	private JButton exit;
	private JCheckBox showPoints;
	private JCheckBox showTraces;
	private JCheckBox trialByTrial;
	private JComboBox scaleCombo;
	private JLabel scaleLabel;

	private JFileChooser fc;
	private File f;
	final Font F16 = new Font("sansserif", Font.PLAIN, 16);

	private boolean traces = true;
	private boolean points = false;
	private boolean trialMode = false;
	
	double scale;

	public FittsTiltTraceFrame(String fileArg)
	{
		fc = new JFileChooser(new File("."));
		final String[] EXTENSIONS = { ".sd3" };
		fc.addChoosableFileFilter(new MyFileFilter(EXTENSIONS));

		tp = new TracePanel(fileArg);
		tp.setBorder(BorderFactory.createLineBorder(Color.gray));

		previous = new JButton("Previous");
		next = new JButton("Next");
		exit = new JButton("Exit");

		showTraces = new JCheckBox("Show traces");
		showPoints = new JCheckBox("Show points");
		trialByTrial = new JCheckBox("Trial Mode");
		showTraces.setSelected(true);
		showPoints.setSelected(false);
		trialByTrial.setSelected(true);
		trialByTrial.setSelected(false);
		
		scaleLabel = new JLabel("Scale: ");
		scaleLabel.setFont(F16);
		
		scaleCombo = new JComboBox();
		for (int i = 0; i < SCALE_OPTIONS.length; ++i)
			scaleCombo.addItem(SCALE_OPTIONS[i]);		
		scaleCombo.setSelectedIndex(DEFAULT_INDEX);
		scaleCombo.setMaximumSize(new Dimension(60, 25));

		scale = Double.parseDouble((String)scaleCombo.getSelectedItem());

		browse = new JButton("Browse...");
		previous.setFont(F16);
		next.setFont(F16);
		exit.setFont(F16);
		browse.setFont(F16);
		showTraces.setFont(F16);
		showPoints.setFont(F16);
		trialByTrial.setFont(F16);
		scaleCombo.setFont(F16);

		file = new JTextField();
		file.setEditable(false);
		file.setFocusable(false);
		file.setBackground(Color.white);
		Dimension d2 = browse.getPreferredSize();
		file.setMaximumSize(new Dimension(400, d2.height));
		file.setFont(F16);
		file.setText(tp.getFileName());

		// -------------
		// add listeners
		// -------------

		next.addActionListener(this);
		previous.addActionListener(this);
		exit.addActionListener(this);
		browse.addActionListener(this);
		file.addActionListener(this);
		this.addKeyListener(this);
		showTraces.addItemListener(this);
		showPoints.addItemListener(this);
		trialByTrial.addItemListener(this);
		scaleCombo.addActionListener(this);

		// ------------------
		// arrange components
		// ------------------

		JPanel p1 = new JPanel();
		p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
		p1.add(scaleLabel);
		p1.add(scaleCombo);
		p1.add(Box.createRigidArea(new Dimension(5, 0)));
		p1.add(showTraces);
		p1.add(showPoints);
		p1.add(trialByTrial);
		p1.add(Box.createRigidArea(new Dimension(20, 0)));
		p1.add(file);
		p1.add(Box.createRigidArea(new Dimension(5, 0)));
		p1.add(browse);
		p1.add(Box.createRigidArea(new Dimension(20, 0)));
		p1.add(previous);
		p1.add(Box.createRigidArea(new Dimension(5, 0)));
		p1.add(next);
		p1.add(Box.createRigidArea(new Dimension(5, 0)));
		p1.add(exit);
		p1.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.add(tp);
		p.add(p1);
		p.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		p.setPreferredSize(new Dimension(d.width, d.height - 45));

		this.pack();
		this.setContentPane(p);
	}

	// -------------------------------
	// implement ItemListener method
	// -------------------------------
	public void itemStateChanged(ItemEvent ie)
	{
		Object source = ie.getSource();
		if (source == showTraces)
			traces = !traces;
		else if (source == showPoints)
			points = !points;
		else if (source == trialByTrial)
		{
			trialMode = !trialMode;
			trialCounter = 0;
		}
		this.repaint();
	}

	// -------------------------------
	// implement ActionListener method
	// -------------------------------

	public void actionPerformed(ActionEvent ae)
	{
		Object source = ae.getSource();

		if (source == next)
			tp.next();
		else if (source == previous)
			tp.previous();
		else if (source == exit)
			System.exit(0);
		else if (source == browse)
		{
			if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				f = fc.getSelectedFile();
				tp.loadData(f);
				file.setText(f.getName());
			}
		} else if (source == file)
		{
			tp.loadData(new File(file.getText()));
		} else if (source == scaleCombo)
		{		
			scale = Double.parseDouble((String)scaleCombo.getSelectedItem());
			tp.repaint();
		}
	}

	public void keyPressed(KeyEvent ke)
	{
	} // do nothing

	public void keyReleased(KeyEvent ke)
	{
	} // do nothing

	public void keyTyped(KeyEvent ke)
	{
		System.out.println(ke.getKeyChar());
	}

	// -----------------------------------------
	// A class to display the targets and traces
	// -----------------------------------------

	class TracePanel extends JPanel implements ActionListener
	{
		static final long serialVersionUID = 42L;
		//final double SCALE = 0.8f;
		// final double SCALE = 1.0f;
		final int BALL_DIAMETER = (int)(20 * scale);

		TrialTrace[] tt;
		String fileName;
		JButton dummy = new JButton();
		String participantCode, selectionMode, tiltGain;

		Timer delay = new Timer(500, this);

		TracePanel(String fileArg)
		{
			// delay.start();
			dummy.addActionListener(this);

			this.setBackground(Color.white);

			// popup file chooser dialog to get sd3 file
			if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				f = fc.getSelectedFile();
				loadData(f);
				fileName = f.getName();
			} else
				fileName = ""; // if Cancel, use null string for filename
		}

		public void actionPerformed(ActionEvent ae)
		{
			// must be from dummy button
			delay.start();
		}

		public void drawTargets(Graphics g)
		{
			double centerCircleX = scale * (tt[0].fromX + tt[0].targetX) / 2.0;
			double centerCircleY = scale * (tt[0].fromY + tt[0].targetY) / 2.0;
			double centerDisplayX = this.getWidth() / 2.0;
			double centerDisplayY = this.getHeight() / 2.0;
			double offsetX = centerDisplayX - centerCircleX;
			double offsetY = centerDisplayY - centerCircleY;

			if (tt == null || tt.length == 0)
				return;

			int sequence = sequenceCounter;
			Graphics2D g2 = (Graphics2D)g;
			g2.setColor(Color.black);
			for (int i = 0; i < tt.length; ++i)
			{
				if (sequence == tt[i].sequence)
				{
					if (tt[i].trialNumber == 1) // "from" target in 1st trial
					{
						double w = scale * tt[i].w;
						double x = offsetX + scale * tt[i].fromX - (w / 2);
						double y = offsetY + scale * tt[i].fromY - (w / 2);
						Ellipse2D.Double e = new Ellipse2D.Double(x, y, w, w);
						g2.draw(e);
					}
					double w = scale * tt[i].w;
					double x = offsetX + scale * tt[i].targetX - (w / 2);
					double y = offsetY + scale * tt[i].targetY - (w / 2);
					Ellipse2D.Double e = new Ellipse2D.Double(x, y, w, w);
					if (trialCounter == tt[i].trialNumber)
					{
						g2.setColor(Color.lightGray);
						g2.fill(e); // fill the target to select in light gray
						g2.setColor(Color.black);
					}
					g2.draw(e);
				}
			}
		}

		public void drawTraces(Graphics g)
		{
			double centerCircleX = scale * (tt[0].fromX + tt[0].targetX) / 2.0;
			double centerCircleY = scale * (tt[0].fromY + tt[0].targetY) / 2.0;
			double centerPanelX = this.getWidth() / 2.0;
			double centerPanelY = this.getHeight() / 2.0;
			double offsetX = centerPanelX - centerCircleX;
			double offsetY = centerPanelY - centerCircleY;

			dummy.doClick();

			if (tt == null || tt.length == 0)
				return;

			int sequence = sequenceCounter;
			Graphics2D g2 = (Graphics2D)g;
			Ellipse2D.Double e;

			for (int i = 0; i < tt.length; ++i)
			{
				if (sequence == tt[i].sequence)
				{
					TracePoint[] tp = tt[i].p;

					// draw special marker at start of sequence
					if (tt[i].trialNumber == 0 && (!trialMode || (trialMode && trialCounter == tt[i].trialNumber)))
					{
						double xx = offsetX + scale * tt[i].p[0].x - 5;
						double yy = offsetY + scale * tt[i].p[0].y - 5;
						e = new Ellipse2D.Double(xx, yy, 11, 11);
						g2.setColor(Color.BLACK);
						g2.draw(e);
						g2.setColor(Color.YELLOW);
						g2.fill(e);
					}

					int j;
					for (j = 0; j < tp.length - 1; ++j)
					{
						double x1 = offsetX + scale * tp[j].x;
						double y1 = offsetY + scale * tp[j].y;
						double x2 = offsetX + scale * tp[j + 1].x;
						double y2 = offsetY + scale * tp[j + 1].y;

						Line2D.Double inkSegment = new Line2D.Double(x1, y1, x2, y2);

						g2.setColor(new Color(0, 0, 128)); // deep blue

						// make the thickness proportional to the tilt magnitude
						float lineWidth = (float)tp[j + 1].tilt / 2f;
						g2.setStroke(new BasicStroke(lineWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));

						if (traces && (!trialMode || (trialMode && trialCounter == tt[i].trialNumber)))
							g2.draw(inkSegment); // draw it!

						Ellipse2D.Double pt = new Ellipse2D.Double(x2 - 1, y2 - 1, 7, 7);
						if (points && (!trialMode || (trialMode && trialCounter == tt[i].trialNumber)))
						{
							g2.draw(pt);
							g2.fill(pt);
						}
					}
					// draw marker at selection point
					double x = offsetX + scale * tp[j].x - 2;
					double y = offsetY + scale * tp[j].y - 2;
					e = new Ellipse2D.Double(x, y, 7, 7);
					g2.setColor(Color.red);
					if (!trialMode || (trialMode && trialCounter == tt[i].trialNumber))
					{
						g2.draw(e);
						g2.fill(e);
					}
				}
			}
		}

		// load the sd3 data into an array tt of the class TrialTrace
		private boolean loadData(File fArg)
		{
			String fileName = fArg.getName();
			String filePath = fArg.getPath();

			// get participant code
			int z = fileName.indexOf("P");
			participantCode = fileName.substring(z, z + 3);

			// first extract selection mode and TVG from file name
			if (fileName.indexOf("FE") > 0)
				selectionMode = "First_Entry";
			else if (fileName.indexOf("DW500") > 0)
				selectionMode = "Dwell_500";
			else
				selectionMode = "?";

			if (fileName.indexOf("TG25") > 0)
				tiltGain = "25";
			else if (fileName.indexOf("TG50") > 0)
				tiltGain = "50";
			else if (fileName.indexOf("TG100") > 0)
				tiltGain = "100";
			else if (fileName.indexOf("TG200") > 0)
				tiltGain = "200";
			else
				tiltGain = "?";

			BufferedReader br = null;
			try
			{
				br = new BufferedReader(new FileReader(filePath));
			} catch (FileNotFoundException e)
			{
				JLabel tmp = new JLabel("File not found: " + filePath);
				tmp.setFont(F16);
				JOptionPane.showMessageDialog(this, tmp, "Error", JOptionPane.ERROR_MESSAGE);
				return false;
			}

			Vector<TrialTrace> v = new Vector<TrialTrace>();
			try
			{
				br.readLine(); // discard 1st header
				br.readLine(); // discard 2nd header
				String tLine = null;
				while ((tLine = br.readLine()) != null)
				{
					String xLine = br.readLine().trim();
					String yLine = br.readLine().trim();
					String tiltLine = br.readLine().trim();

					// delete commas at end of lines (if any)
					if (tLine.charAt(tLine.length() - 1) == ',')
						tLine = tLine.substring(0, tLine.length() - 1);
					if (xLine.charAt(xLine.length() - 1) == ',')
						xLine = xLine.substring(0, xLine.length() - 1);
					if (yLine.charAt(yLine.length() - 1) == ',')
						yLine = yLine.substring(0, yLine.length() - 1);
					if (tiltLine.charAt(tiltLine.length() - 1) == ',')
						tiltLine = tiltLine.substring(0, tiltLine.length() - 1);

					StringTokenizer stt = new StringTokenizer(tLine, ",");
					StringTokenizer stx = new StringTokenizer(xLine, ",");
					StringTokenizer sty = new StringTokenizer(yLine, ",");
					StringTokenizer stTilt = new StringTokenizer(tiltLine, ",");

					int sequence = Integer.parseInt(stx.nextToken()); // sequence
					double a = Double.parseDouble(stx.nextToken()); // a
					double w = Double.parseDouble(stx.nextToken()); // w
					int trialNumber = Integer.parseInt(stx.nextToken()); // trial
					double x1 = Double.parseDouble(stx.nextToken()); // from_x
					double y1 = Double.parseDouble(stx.nextToken()); // from_y
					double x2 = Double.parseDouble(stx.nextToken()); // to_x
					double y2 = Double.parseDouble(stx.nextToken()); // to_y
					stx.nextToken(); // "x=" (trace data begins with next token

					sty.nextToken(); // skip over the lead-in info for the y data
					sty.nextToken();
					sty.nextToken();
					sty.nextToken();
					sty.nextToken();
					sty.nextToken();
					sty.nextToken();
					sty.nextToken();
					sty.nextToken();

					stt.nextToken(); // skip over the lead-in info for the t data
					stt.nextToken();
					stt.nextToken();
					stt.nextToken();
					stt.nextToken();
					stt.nextToken();
					stt.nextToken();
					stt.nextToken();
					stt.nextToken();

					stTilt.nextToken(); // skip over the lead-in info for the tilt data
					stTilt.nextToken();
					stTilt.nextToken();
					stTilt.nextToken();
					stTilt.nextToken();
					stTilt.nextToken();
					stTilt.nextToken();
					stTilt.nextToken();
					stTilt.nextToken();

					// int[] timeStamp = new int[stt.countTokens()];
					int n = stt.countTokens();
					TracePoint[] tp = new TracePoint[stt.countTokens()];

					for (int i = 0; i < n; ++i)
					{
						int t = Integer.parseInt(stt.nextToken());
						double x = Double.parseDouble(stx.nextToken());
						double y = Double.parseDouble(sty.nextToken());
						double tilt = Double.parseDouble(stTilt.nextToken());
						tp[i] = new TracePoint(x, y, t, tilt);
					}
					v.add(new TrialTrace(sequence, trialNumber, a, w, x1, y1, x2, y2, BALL_DIAMETER, tp));
				}
			} catch (IOException e)
			{
				JLabel tmp = new JLabel("Data format error in file: " + fileName);
				tmp.setFont(F16);
				JOptionPane.showMessageDialog(this, tmp, "Error", JOptionPane.ERROR_MESSAGE);
				return false;
			}
			tt = new TrialTrace[v.size()];
			v.copyInto(tt);

			sequenceCounter = 0;
			trialCounter = 0;
			this.repaint();
			return true;
		}

		public String getFileName()
		{
			return fileName;
		}

		// return the number of sequences of trials (in this file/block)
		int numberOfSequences()
		{
			if (tt == null || tt.length == 0) // no data
				return -1;
			else
				return tt.length / trialsPerSequence();
		}

		// return the number of trials in the current sequence
		int trialsPerSequence()
		{
			if (tt == null || tt.length == 0) // no data
				return -1;
			else
				return tt[tt.length - 1].trialNumber + 1;
		}

		public void clear()
		{
			this.repaint();
		}

		public void paintComponent(Graphics g)
		{
			super.paintComponent(g); // paint background
			paintBlock(g);
		}

		private void paintBlock(Graphics g)
		{
			this.paintInfo(g);
			this.drawTargets(g);
			this.drawTraces(g);
		}

		private void paintInfo(Graphics g)
		{
			String header1 = "Sequence: " + (sequenceCounter + 1) + " of " + numberOfSequences() + " (A="
					+ Math.round(this.getA(sequenceCounter)) + ", W=" + Math.round(this.getW(sequenceCounter)) + ")";
			String header2 = trialMode ? "Trial: " + (trialCounter + 1) + " of " + trialsPerSequence()
					: "Trial: (all trials)";

			Graphics2D g2 = (Graphics2D)g;
			g2.setColor(new Color(128, 0, 0));
			g2.setFont(new Font("SansSerif", Font.PLAIN, 14));
			FontMetrics fm = g2.getFontMetrics();
			int height = fm.getHeight();

			g2.drawString("Participant = " + participantCode, 10, height);
			g2.drawString("Selection mode = " + selectionMode, 10, 2 * height);
			g2.drawString("Tilt gain = " + tiltGain, 10, 3 * height);
			g2.drawString(header1, 10, 4 * height);
			g2.drawString(header2, 10, 5 * height);
			g2.drawString("-----", 10, 6 * height);

			if (trialMode)
			{
				int idx = sequenceCounter * trialsPerSequence() + trialCounter;
				g2.drawString("Number of points = " + tt[idx].numberOfSamplePoints, 10, 7 * height);
				g2.drawString("PT = " + tt[idx].positioningTime + " ms", 10, 8 * height);
				g2.drawString("ST = " + tt[idx].selectionTime + " ms", 10, 9 * height);
				g2.drawString("MT = " + tt[idx].movementTime + " ms", 10, 10 * height);
				g2.drawString(String.format("Max Tilt = %1.1f degrees", tt[idx].maxTilt), 10, 11 * height);
				g2.drawString("TRE = " + tt[idx].tre, 10, 12 * height);
				g2.drawString("TAC = " + tt[idx].tac, 10, 13 * height);
				g2.drawString("MDC = " + tt[idx].mdc, 10, 14 * height);
				g2.drawString("ODC = " + tt[idx].odc, 10, 15 * height);
				g2.drawString(String.format("MV = %1.2f", tt[idx].mv), 10, 16 * height);
				g2.drawString(String.format("ME = %1.2f", tt[idx].me), 10, 17 * height);
				g2.drawString(String.format("MO = %1.2f", tt[idx].mo), 10, 18 * height);
			} else
			{
				int baseIdx = sequenceCounter * trialsPerSequence();
				int ptSequence = 0;
				int stSequence = 0;
				int mtSequence = 0;
				float maxTiltSequence = 0.0f;
				int treSequence = 0;
				int tacSequence = 0;
				int mdcSequence = 0;
				int odcSequence = 0;
				float mvSequence = 0.0f;
				float meSequence = 0.0f;
				float moSequence = 0.0f;
				for (int i = 0; i < trialsPerSequence(); ++i)
				{
					ptSequence += tt[baseIdx + i].positioningTime;
					stSequence += tt[baseIdx + i].selectionTime;
					mtSequence += tt[baseIdx + i].movementTime;
					maxTiltSequence += tt[baseIdx + i].maxTilt;
					treSequence += tt[baseIdx + i].tre;
					tacSequence += tt[baseIdx + i].tac;
					mdcSequence += tt[baseIdx + i].mdc;
					odcSequence += tt[baseIdx + i].odc;
					mvSequence += tt[baseIdx + i].mv;
					meSequence += tt[baseIdx + i].me;
					moSequence += tt[baseIdx + i].mo;
				}
				ptSequence /= trialsPerSequence(); // these stats reported as means
				stSequence /= trialsPerSequence();
				mtSequence /= trialsPerSequence();
				maxTiltSequence /= trialsPerSequence();
				mvSequence /= trialsPerSequence();
				meSequence /= trialsPerSequence();
				moSequence /= trialsPerSequence();

				g2.drawString("Performance measures (mean per sequence)", 10, 7 * height);
				g2.drawString("PT = " + ptSequence + " ms", 10, 8 * height);
				g2.drawString("ST = " + stSequence + " ms", 10, 9 * height);
				g2.drawString("MT = " + mtSequence + " ms", 10, 10 * height);
				g2.drawString(String.format("Max Tilt = %1.1f degrees", maxTiltSequence), 10, 11 * height);
				g2.drawString("TRE = " + treSequence, 10, 12 * height);
				g2.drawString("TAC = " + tacSequence, 10, 13 * height);
				g2.drawString("MDC = " + mdcSequence, 10, 14 * height);
				g2.drawString("ODC = " + odcSequence, 10, 15 * height);
				g2.drawString(String.format("MV = %1.2f", mvSequence), 10, 16 * height);
				g2.drawString(String.format("ME = %1.2f", meSequence), 10, 17 * height);
				g2.drawString(String.format("MO = %1.2f", moSequence), 10, 18 * height);
			}
		}

		// return the target width for the corresponding sequence
		double getW(int sequenceArg)
		{
			double w = -1.0;
			if (tt == null || tt.length == 0) // no data, return now
				return w;

			for (int i = 0; i < tt.length; ++i)
			{
				if (sequenceArg == tt[i].sequence)
				{
					w = tt[i].w;
					break;
				}
			}
			//return (w * 1.0 / scale); // undo scaling
			return w;
		}

		// return the target amplitude for the corresponding sequence
		double getA(int sequenceArg)
		{
			double a = -1.0;
			if (tt == null || tt.length == 0) // no data, return now
				return a;

			for (int i = 0; i < tt.length; ++i)
			{
				if (sequenceArg == tt[i].sequence)
				{
					a = tt[i].a;
					break;
				}
			}
			//return (a * 1.0 / scale); // undo scaling
			return a;
		}

		// display trace data for the next sequence/trial
		public void next()
		{
			if (trialMode)
			{
				++trialCounter;
				if (trialCounter >= trialsPerSequence())
					trialCounter = 0;
			} else
			{
				++sequenceCounter;
				if (sequenceCounter >= numberOfSequences())
					sequenceCounter = 0;
			}
			this.repaint();
		}

		// display trace data for previous sequence/trial
		public void previous()
		{
			if (trialMode)
			{
				--trialCounter;
				if (trialCounter < 0)
					trialCounter = trialsPerSequence() - 1;
			} else
			{
				--sequenceCounter;
				if (sequenceCounter < 0)
					sequenceCounter = numberOfSequences() - 1;
			}
			this.repaint();
		}
	}

	/**
	 * A class to extend the FileFilter class (which is abstract) and implement the 'accept' and 'getDescription'
	 * methods.
	 */
	class MyFileFilter extends FileFilter
	{
		private String[] s;

		MyFileFilter(String[] sArg)
		{
			s = sArg;
		}

		// determine which files to display in the chooser
		public boolean accept(File fArg)
		{
			// if it's a directory, show it
			if (fArg.isDirectory())
				return true;

			// if the filename contains the extension, show it
			for (int i = 0; i < s.length; ++i)
				if (fArg.getName().toLowerCase().indexOf(s[i].toLowerCase()) > 0)
					return true;

			// filter out everything else
			return false;
		}

		public String getDescription()
		{
			String tmp = "";
			for (int i = 0; i < s.length; ++i)
				tmp += "*" + s[i] + " ";

			return tmp;
		}
	}
}